@@ -19,12 +19,19 @@ var LibraryFetch = {
     return _fetch_work_queue;
   },
 
+#if FETCH_SUPPORT_INDEXEDDB
   $__emscripten_fetch_delete_cached_data: __emscripten_fetch_delete_cached_data,
   $__emscripten_fetch_load_cached_data: __emscripten_fetch_load_cached_data,
   $__emscripten_fetch_cache_data: __emscripten_fetch_cache_data,
+#endif
   $__emscripten_fetch_xhr: __emscripten_fetch_xhr,
-  emscripten_start_fetch__deps: ['$Fetch', '$__emscripten_fetch_xhr', '$__emscripten_fetch_cache_data', '$__emscripten_fetch_load_cached_data', '$__emscripten_fetch_delete_cached_data', '_emscripten_get_fetch_work_queue', 'emscripten_is_main_runtime_thread'],
-  emscripten_start_fetch: emscripten_start_fetch
+
+  emscripten_start_fetch: emscripten_start_fetch,
+  emscripten_start_fetch__deps: ['$Fetch', '$__emscripten_fetch_xhr',
+#if FETCH_SUPPORT_INDEXEDDB
+  '$__emscripten_fetch_cache_data', '$__emscripten_fetch_load_cached_data', '$__emscripten_fetch_delete_cached_data',
+#endif
+  '_emscripten_get_fetch_work_queue', 'emscripten_is_main_runtime_thread']
 };
 
 mergeInto(LibraryManager.library, LibraryFetch);
